#!/bin/sh
# find-debuginfo.sh - automagically generate debug info and file list
# for inclusion in an rpm spec file.

if [ -z "$1" ]; then
   	BUILDDIR="."
else
   	BUILDDIR=$1
fi

LISTFILE=$BUILDDIR/debugfiles.list
SOURCEFILE=$BUILDDIR/debugsources.list
DEBUGFILES=$BUILDDIR/debugfiles-add.list

: > $SOURCEFILE
touch $LISTFILE

strip_to_debug() {
  objcopy --only-keep-debug --remove-section .comment "$2" "$1"
  objcopy --add-gnu-debuglink="$1" "$2"
}

extract_debuginfo() {
	:
}

if [ ! -d $RPM_BUILD_ROOT ]; then
	# no buildroot, exit
	exit 0
fi

if [ -f $DEBUGFILES ]; then
	cat $DEBUGFILES > $LISTFILE
fi

filelist=$(find $RPM_BUILD_ROOT ! -path "$RPM_BUILD_ROOT/usr/lib/debug/*.debug" -type f '(' -perm -0100 -or -perm -0010 -or -perm -0001 ')')
if [ -z "$filelist" -a ! -f $DEBUGFILES ]; then
	# no files, exit
	exit 0
fi

filetypes=$(echo "$filelist" | xargs -r -d'\n' file)
elflist=$(echo "$filetypes" | awk -F: '/ELF.*, not stripped/ {print $1}')
if [ -z "$elflist" -a ! -f $DEBUGFILES ]; then
	# no elf objects, exit
	exit 0
fi

# Strip ELF binaries
echo "$elflist" | while read f; do
	dn=$(dirname "${f#$RPM_BUILD_ROOT}")
	bn=$(basename "$f" .debug).debug

	debugdn="$RPM_BUILD_ROOT/usr/lib/debug$dn"
	debugfn="$debugdn/$bn"
	[ -f "$debugfn" ] && continue

	echo extracting debug info from $f
	/usr/lib/rpm/debugedit -b "$RPM_BUILD_DIR" -d /usr/src/debug -l "$SOURCEFILE" "$f"

	# A binary already copied into /usr/lib/debug doesn't get stripped,
	# just has its file names collected and adjusted.
	case "$dn" in
	/usr/lib/debug/*) continue ;;
	esac

	mkdir -p "$debugdn"
	if test -w "$f"; then
		strip_to_debug "$debugfn" "$f"
	else
		chmod u+w "$f"
		strip_to_debug "$debugfn" "$f"
		chmod u-w "$f"
	fi
done

mkdir -p $RPM_BUILD_ROOT/usr/src/debug
cat $SOURCEFILE | (cd $RPM_BUILD_DIR; LANG=C sort -z -u | fgrep -zv '<built-in>' | cpio --quiet -pd0mL $RPM_BUILD_ROOT/usr/src/debug)
# stupid cpio creates new directories in mode 0700, fixup
find $RPM_BUILD_ROOT/usr/src/debug -type d -print0 | xargs -0 chmod a+rx

find $RPM_BUILD_ROOT/usr/lib/debug -type f | sed -n -e "s#^$RPM_BUILD_ROOT##p" > $LISTFILE
if [ -f $DEBUGFILES ]; then
	sed -e "s#^$RPM_BUILD_ROOT##" $DEBUGFILES >> $LISTFILE
fi
find $RPM_BUILD_ROOT/usr/lib/debug -mindepth 1 -type d | sed -n -e "s#^$RPM_BUILD_ROOT#%dir #p" >> $LISTFILE
find $RPM_BUILD_ROOT/usr/src/debug -mindepth 1 -maxdepth 1 | sed -n -e "s#^$RPM_BUILD_ROOT##p" >> $LISTFILE
